import * as Mqtt from 'mqtt'
import * as Mongo from 'mongodb'
import * as Queue from './queue'
import * as MqttMessage from './mqtt_message'


var MQTT_BROKER_URL : String = 'mqtt://localhost'
var MONGO_DATABASE_URL : String = 'mongodb://localhost/test'
var COLLECTION : String = 'mqttMongo'
var TOPIC : String = 'mqttMongo'

import options = require('commander')
options
  .version('0.0.1')
  .option('-b, --broker [broker url]', 'Broker URL', MQTT_BROKER_URL)
  .option('-m, --database [database url]', 'MongoDB Database URL', MONGO_DATABASE_URL)
  .option('-c, --collection [mongodb collection]', 'Collection in MongoDB Database', COLLECTION)
  .option('-t, --topic [topic to subscribe]', 'Topic to subscribe', TOPIC)
  .parse(process.argv)



var dbHandle : Mongo.Db;
var dbReady : boolean = false;

Mongo.MongoClient.connect(options['database'], 
  { 
    'server': {
      'reconnectTries': 10000,
      'reconnectInterval': 1000,
      'socketOptions': {
        'autoReconnect': false
      }
    }  
  })
  .then(
    (tmpDbHandle: Mongo.Db) => { 
      dbHandle = tmpDbHandle
      dbReady = true; 
      console.log("Connected to database")
      dbHandle.on('error', () => { console.log("error on database") })
      dbHandle.on('reconnect', () => { 
        console.log("reconnect on database") 
        dbReady = true
        queue.knock()
      })
      dbHandle.on('timeout', () => { console.log("timeout on database") })
      dbHandle.on('close', () => { 
        console.log("close on database") 
        dbReady = false
      })
    },
    (err: String) => {
      console.log("Unable to connect to database: %s", err)
      process.exit(1)
    }
)


var queue = new Queue.Queue<MqttMessage.MqttMessage>()
queue.on('data', () => {
  if (dbReady) {
    while (! queue.isEmpty()) {
      var msg : MqttMessage.MqttMessage = queue.deq()
      console.log("Something in the queue: %s", JSON.stringify(msg))
      var coll = dbHandle.collection(options['collection'])
      coll.insertOne(msg.getMessage())
        .then(
          (r) => { 
            console.log("successfully inserted into database")
          },
          (err) => { 
            console.log("error when trying to insert into database")
            queue.reenq(msg)
          }
        )
    }
  } else {
    console.log("database currently not available, not reading from stream")
  }
})


var mqttClient = Mqtt.connect(options['broker'])
mqttClient.on('offline', () => { console.log("mqtt client is offline") })
mqttClient.on('reconnect', () => { console.log("mqtt client is reconnecting") })
mqttClient.on('close', () => { console.log("mqtt connection closed") })

mqttClient.on('connect', () => {
  console.log("mqtt client connected to broker")
  mqttClient.subscribe(options['topic'])
  mqttClient.subscribe('MqttMongo/Command')
  mqttClient.publish('MqttMongo/Status', 'hello, started up')
})

var msgCnt : number = 0
mqttClient.on('message', (topic : string, message : string) => {
  msgCnt++;
  console.log(`message received ${msgCnt}, topic ${topic}, payload ${message}`)

  if (topic == "MqttMongo/Command" && message == "shutdown") {
    clearInterval(uptimeInterval)
    mqttClient.end()
    dbHandle.close()
  } else {
    var mqttMessage = new MqttMessage.MqttMessage(topic, message)
    queue.enq(mqttMessage)
  }
})


var uptime : number = 0
var uptimeInterval = setInterval(() => {
  uptime++
  mqttClient.publish('MqttMongo/Status', `{'Uptime': ${uptime}}`)
}, 1000)



console.log("MqttMongo started")